__title__ = 'qartez.defaults'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__all__ = (
    'PREPEND_LOC_URL_WITH_SITE_URL', 'PREPEND_IMAGE_LOC_URL_WITH_SITE_URL',
    'CHANGEFREQ', 'DEBUG'
)

# When set to True, current site's domain is prepended to the location URL.
PREPEND_LOC_URL_WITH_SITE_URL = True

# When set to True, current site's domain is prepended to the image location
# URL.
PREPEND_IMAGE_LOC_URL_WITH_SITE_URL = True

# Valid changefreq values according to the specs
# http://www.sitemaps.org/protocol.html
CHANGEFREQ = [
    'always', 'hourly', 'daily', 'weekly', 'monthly', 'yearly', 'never'
    ]

DEBUG = False



"""
Settings module.

Override the following values in your global ``settings`` module by adding
`QARTEZ_` prefix to the values.

When it comes to importing the values, import them from ``qartez.settings``
module (without `QARTEZ_` prefix).

``PREPEND_LOC_URL_WITH_SITE_URL``: When set to True, current site's domain is
prepended to the location URL.

``PREPEND_IMAGE_LOC_URL_WITH_SITE_URL``: When set to True, current site's
domain is prepended to the image location URL.

``CHANGEFREQ``: Valid changefreq values according to the specs
http://www.sitemaps.org/protocol.html
"""

from .conf import get_setting

__title__ = 'qartez.settings'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__all__ = (
    'PREPEND_LOC_URL_WITH_SITE_URL', 'PREPEND_IMAGE_LOC_URL_WITH_SITE_URL',
    'CHANGEFREQ', 'DEBUG',
)

PREPEND_LOC_URL_WITH_SITE_URL = get_setting('PREPEND_LOC_URL_WITH_SITE_URL')
PREPEND_IMAGE_LOC_URL_WITH_SITE_URL = get_setting(
    'PREPEND_IMAGE_LOC_URL_WITH_SITE_URL'
    )
CHANGEFREQ = get_setting('CHANGEFREQ')

DEBUG = get_setting('DEBUG')



from django.http import HttpResponse, Http404
from django.template import loader
from django.utils.encoding import smart_str
from django.core.paginator import EmptyPage, PageNotAnInteger

__title__ = 'qartez.views'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__all__ = ('render_images_sitemap',)


def render_images_sitemap(request, sitemaps, section=None,
                          template_name='qartez/images_sitemap.xml'):
    """Render images sitemap.

    :param django.http.HttpRequest request:
    :param sitemaps:
    :param secion:
    :param str template_name:
    :return django.http.HttpResponse:
    """
    maps, urls = [], []
    if section is not None:
        if section not in sitemaps:
            raise Http404(
                "No sitemap available for section: {0}".format(section)
                )
        maps.append(sitemaps[section])
    else:
        maps = sitemaps.values()
    page = request.GET.get("p", 1)
    for site in maps:
        try:
            if callable(site):
                urls.extend(site().get_urls(page))
            else:
                urls.extend(site.get_urls(page))
        except EmptyPage:
            raise Http404("Page {0} empty".format(page))
        except PageNotAnInteger:
            raise Http404("No page {0}".format(page))
    xml = smart_str(loader.render_to_string(
        template_name, {'urlset': urls, 'request': request})
        )
    try:
        return HttpResponse(xml, mimetype='application/xml')
    except TypeError:
        return HttpResponse(xml, content_type='application/xml')



from __future__ import unicode_literals

import os
import unittest

import pytest

__title__ = 'qartez.tests.test_sitemaps'
__author__ = 'Artur Barseghyan'
__copyright__ = '2013-2017 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'


# Skipping from non-Django tests.
if os.environ.get("DJANGO_SETTINGS_MODULE", None):
    from django.test import Client
    from django.test import TestCase

    from qartez.tests.base import print_info

    import factories

    @pytest.mark.django_db
    class QartezTest(TestCase):
        """
        Testing sitemaps.
        """

        # fixtures = ['initial_data.json',]

        pytestmark = pytest.mark.django_db

        def setUp(self):
            # Testing the URLs
            self.urls = {
                'sitemap of sitemaps': '/sitemap.xml',
                'normal sitemap': '/sitemap-foo-items.xml',
                'images sitemap': '/sitemap-foo-images.xml',
                'static sitemap': '/sitemap-foo-static.xml',
                'alternative hreflang sitemap':
                    '/sitemap-foo-items-alternate-hreflang.xml',
                'images custom sitemap': '/sitemap-foo-images-custom.xml',
            }

            factories.FooItemFactory.create_batch(100)

        @print_info
        def test_all_sitemaps(self):
            """
            Test the all sitemaps.
            """
            flow = []
            # Testing view with signed URL
            client = Client()
            for description, url in self.urls.items():
                response = client.get(url, {})
                self.assertTrue(response.status_code in (200, 201, 202))
                flow.append(
                    'Response status code for {0} is {1}'.format(
                        description, response.status_code
                    )
                )

            return flow

        @print_info
        def test_01_alternative_hreflang_sitemap(self):
            """Test alternate hreflang sitemap."""
            flow = []
            c = Client()
            response = c.get('/sitemap-foo-items-alternate-hreflang.xml', {})
            response_content = response.content.decode()
            # import ipdb; ipdb.set_trace()
            self.assertTrue('hreflang="en-us"' in response_content)
            self.assertTrue('rel="alternate"' in response_content)
            # self.assertTrue(b'hreflang="en-us"' in response.content)

        @print_info
        def test_02_static_sitemap(self):
            """Test static sitemap."""
            flow = []
            c = Client()
            response = c.get('/sitemap-foo-static.xml', {})
            response_content = response.content.decode()
            self.assertTrue('http://example.com/foo/'
                            'contact/' in response_content)

        @print_info
        def test_03_images_sitemap(self):
            """Test images sitemap."""
            flow = []
            c = Client()
            response = c.get('/sitemap-foo-images.xml', {})
            response_content = response.content.decode()
            self.assertTrue('http://www.google.com/'
                            'schemas/sitemap-image/1.1' in response_content)
            self.assertTrue('<image:title>' in response_content)
            self.assertTrue('<image:loc>' in response_content)
            self.assertTrue('<image:image>' in response_content)

        @print_info
        def test_04_sitemap_of_sitemaps(self):
            """Test sitemap of sitemaps."""
            flow = []
            c = Client()
            response = c.get('/sitemap.xml', {})
            response_content = response.content.decode()
            self.assertTrue('<sitemapindex xmlns="'
                            'http://www.sitemaps.org/'
                            'schemas/sitemap/0.9">' in response_content)
            self.assertTrue('https://example.com/'
                            'sitemap-foo-items-alternate-'
                            'hreflang.xml' in response_content)
            self.assertTrue('http://example.com/sitemap-foo-'
                            'items.xml' in response_content)


if __name__ == "__main__":
    # Tests
    unittest.main()



from __future__ import print_function

PRINT_INFO = True

__title__ = 'qartez.tests.base'
__author__ = 'Artur Barseghyan'
__copyright__ = 'Copyright (c) 2013-2014 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'


def print_info(func):
    """Print some useful info."""
    if not PRINT_INFO:
        return func

    def inner(self, *args, **kwargs):
        result = func(self, *args, **kwargs)

        print('\n\n%s' % func.__name__)
        print('============================')
        if func.__doc__:
            print('""" %s """' % func.__doc__.strip())
        print('----------------------------')
        if result is not None:
            print(result)
        print('\n++++++++++++++++++++++++++++')

        return result
    return inner



import datetime

from six import PY3
from six.moves.urllib import parse as urlparse

from django.contrib.sitemaps import Sitemap, GenericSitemap

from django.contrib.sites.models import Site
from django.core.exceptions import ImproperlyConfigured

from .constants import REL_ALTERNATE_HREFLANG_SITEMAP_TEMPLATE
from .settings import (
    PREPEND_LOC_URL_WITH_SITE_URL, PREPEND_IMAGE_LOC_URL_WITH_SITE_URL,
    CHANGEFREQ
)

from nine import versions

if versions.DJANGO_GTE_1_11:
    from django.urls import reverse_lazy
else:
    from django.core.urlresolvers import reverse_lazy

PY2 = not PY3

__title__ = 'django-qartez'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__all__ = (
    'ImagesSitemap',
    'StaticSitemap',
    'RelAlternateHreflangSitemap',
)


class ImagesSitemap(GenericSitemap):
    """Class for image sitemap.

    Implemented according to specs specified by Google
    http://www.google.com/support/webmasters/bin/answer.py?answer=178636

    :example:
    >>> from qartez.sitemaps import ImagesSitemap
    >>>
    >>> foo_item_images_info_dict = {
    >>>     'queryset': FooItem._default_manager.exclude(image=None), # qs
    >>>     'image_location_field': 'image', # image location
    >>>     'image_title_field': 'title', # image title
    >>>     'location_field': 'get_absolute_url' # an absolute URL of the page
    >>>                                          # where image is shown
    >>> }
    >>>
    >>> foo_item_images_sitemap = {
    >>>     'foo_item_images': ImagesSitemap(foo_item_images_info_dict, \
    >>>                                      priority=0.6),
    >>> }
    """

    def __init__(self, info_dict, priority=None, changefreq=None):
        """Constructor.

        :param dict info_dict:
        :param priority float:
        :param str changefreq:
        """
        self.image_location_field = info_dict.get('image_location_field', None)
        self.image_caption_field = info_dict.get('image_caption_field', None)
        self.image_title_field = info_dict.get('image_title_field', None)

        if self.image_title_field:
            if PY2:
                self.image_title_field = unicode(self.image_title_field)
            else:
                self.image_title_field = str(self.image_title_field)

        self.image_geo_location_field = info_dict.get(
            'image_geo_location_field', None
        )
        self.image_license_field = info_dict.get('image_license_field', None)
        self.location_field = info_dict.get('location_field', None)
        super(ImagesSitemap, self).__init__(info_dict, priority, changefreq)

    def image_location(self, item):
        """Get image location."""
        if self.image_location_field is not None:
            try:
                image_location_field = getattr(item, self.image_location_field)

                if callable(image_location_field):
                    return image_location_field()
                else:
                    return image_location_field
            except Exception as e:
                return None

        return None

    def image_caption(self, item):
        """Get image caption."""
        if self.image_caption_field is not None:
            return getattr(item, self.image_caption_field)
        return None

    def image_title(self, item):
        """Get image title."""
        if self.image_title_field is not None:
            return getattr(item, self.image_title_field)
        return None

    def image_geo_location(self, item):
        """Get image geo location."""
        if self.image_geo_location_field is not None:
            return getattr(item, self.image_geo_location_field)
        return None

    def image_license(self, item):
        """Get image geo location."""
        if self.image_license_field is not None:
            return getattr(item, self.image_license_field)
        return None

    def location(self, item):
        """
        Gets image location URL.
        """
        if self.location_field is not None:
            try:
                location_field = getattr(item, self.location_field)
                if callable(location_field):
                    return location_field()
                else:
                    return location_field
            except Exception as e:
                return None
        return item.get_absolute_url()

    def __get(self, name, obj, default=None):
        """Get method."""
        try:
            attr = getattr(self, name)
        except AttributeError:
            return default
        if callable(attr):
            return attr(obj)
        return attr

    def get_urls(self, page=1, site=None, protocol=None):
        """Get URLs."""
        # Determine protocol
        if self.protocol is not None:
            protocol = self.protocol
        if protocol is None:
            protocol = 'http'

        # Determine domain
        if site is None:
            if Site._meta.installed:
                try:
                    site = Site.objects.get_current()
                except Site.DoesNotExist:
                    pass
            if site is None:
                raise ImproperlyConfigured(
                    "To use sitemaps, either enable the sites framework or "
                    "pass a Site/RequestSite object in your view."
                )
        domain = site.domain

        urls = []
        for item in self.paginator.page(page).object_list:
            loc = self.__get('location', item, None)
            if loc and PREPEND_LOC_URL_WITH_SITE_URL:
                if PY2:
                    loc = "{0}://{1}{2}".format(
                        protocol, unicode(domain), unicode(loc)
                    )
                else:
                    loc = "{0}://{1}{2}".format(
                        protocol,
                        str(domain),
                        str(loc)
                    )

            image_loc = self.__get('image_location', item, None)
            if image_loc and PREPEND_IMAGE_LOC_URL_WITH_SITE_URL:
                try:
                    if PY2:
                        image_loc = "{0}://{1}{2}".format(
                            protocol, unicode(domain), unicode(image_loc)
                        )
                    else:
                        image_loc = "{0}://{1}{2}".format(
                            protocol, str(domain), str(image_loc)
                        )
                except Exception as e:
                    continue

            # Validating the changefreq
            changefreq = self.__get('changefreq', item, None)
            if changefreq is not None:
                assert changefreq in CHANGEFREQ

            # Validating the priority
            priority = self.__get('priority', item, None)
            if priority is not None:
                assert priority >= 0 and priority <= 1

            url_info = {
                'location': loc,
                'image_location': image_loc,
                'image_caption': self.__get('image_caption', item, None),
                'image_title': self.__get('image_title', item, None),
                'image_license': self.__get('image_license', item, None),
                'image_geo_location': self.__get(
                    'image_geo_location', item, None
                ),
                'lastmod': self.__get('lastmod', item, None),
                'changefreq': changefreq,
                'priority': priority
            }
            urls.append(url_info)
        return urls


class StaticSitemap(Sitemap):
    """Sitemap for ``static`` pages.

    See constructor docstring for list of accepted (additional) arguments.

    :example:
    >>> from qartez.sitemaps import StaticSitemap
    >>> service_sitemap = StaticSitemap(priority=0.1, changefreq='never')
    >>> service_sitemap.add_named_pattern('blog.welcome')
    >>> service_sitemap.add_named_pattern('feedback.contact')
    >>>
    >>> content_types_sitemap = StaticSitemap(priority=1.0, changefreq='daily')
    >>> content_types_sitemap.add_named_pattern('blog.browse') # Homepage
    >>> content_types_sitemap.add_named_pattern(
    >>>     'blog.browse', kwargs={'content_type': 'articles'}
    >>>     ) # Articles
    >>> content_types_sitemap.add_named_pattern(
    >>>     'blog.browse', kwargs={'content_type': 'downloads'}
    >>>     ) # Downloads
    """
    NAMED_PATTERN = 1
    URL = 2

    def __init__(self, *args, **kwargs):
        """Constructor. Accepts the following optional keyword-arguments (to
        be only specified as keyword-arguments).

        :param float priority:
        :param str changefreq:
        :param datetime.datetime|str lastmod:
        """
        if 'priority' in kwargs:
            self.priority = kwargs.pop('priority')
        else:
            self.priority = 1.0

        if 'changefreq' in kwargs:
            self.changefreq = kwargs.pop('changefreq')
        else:
            self.changefreq = 'never'

        if 'lastmod' in kwargs:
            self.lastmod = kwargs.pop('lastmod')
        else:
            self.lastmod = datetime.datetime.now()

        super(StaticSitemap, self).__init__(*args, **kwargs)
        self._items = []

    def items(self):
        """Return sitemap items.

        :return list:
        """
        return self._items

    def location(self, obj):
        """Location."""
        return obj['location']

    def add_named_pattern(self,
                          viewname,
                          urlconf=None,
                          args=[],
                          kwargs=None,
                          lastmod=None,
                          changefreq=None,
                          priority=None):
        """Ad a named pattern to the items list.

        :param str viewname:
        :param urlconf:
        :param list args:
        :param dict kwargs:
        :param lastmod:
        :param str changefreq:
        :param float priority:
        """
        try:
            loc = reverse_lazy(viewname, urlconf, args, kwargs)
            self._items.append({
                'location': loc,
                'lastmod': lastmod or self.lastmod,
                'changefreq': changefreq if changefreq else self.changefreq,
                'priority': priority if priority else self.priority
            })
        except Exception as e:
            pass

    def add_url(self, url, lastmod=None, changefreq=None, priority=None):
        """Add a URL to the items list.

        :param str url:
        :param lastmod:
        :param str changefreq:
        :param float priority:
        """
        try:
            self.items.append({
                'location': url,
                'lastmod': lastmod or self.lastmod,
                'changefreq': changefreq if changefreq else self.changefreq,
                'priority': priority if priority else self.priority
            })
        except Exception as e:
            pass

    def get_urls(self, *args, **kwargs):
        """Make sure nothing breaks if some URL is unresolvable.

        :return list:
        """
        try:
            return super(StaticSitemap, self).get_urls(*args, **kwargs)
        except Exception as e:
            return []


class RelAlternateHreflangSitemap(Sitemap):
    """Sitemaps: rel="alternate" hreflang="x" implementation.

    Read the specs the specs here
    http://support.google.com/webmasters/bin/answer.py?hl=en&answer=2620865

    IMPORTANT: When you use this class you have to override
    the ``alternate_hreflangs`` method in your sitemap class.

    :example:
    >>> from qartez.sitemaps import RelAlternateHreflangSitemap
    >>>
    >>> class ArticleSitemap(RelAlternateHreflangSitemap):
    >>>     def alternate_hreflangs(self, obj):
    >>>         return [('en-us', obj.alternative_object_url),]
    """

    def __get(self, name, obj, default=None):
        """Get."""
        try:
            attr = getattr(self, name)
        except AttributeError:
            return default
        if callable(attr):
            return attr(obj)
        return attr

    def alternate_hreflangs(self, item):
        """Alternate hreflangs.

        You should override the "alternate_hreflangs" method in your sitemap
        class.
        """
        raise NotImplementedError(
            """You have to override the "alternate_hreflangs" method in """
            """your sitemap class. Refer to "qartez" app documentation for """
            """details and examples."""
        )

    def _full_url(self, protocol, domain, path):
        """Full URL."""
        return "{0}://{1}{2}".format(protocol, domain, path)

    def _render_alternate_hreflangs(self, protocol, domain, item):
        """Render alternative hreflangs.

        Render the tiny bit of XML responsible for rendering the alternate
        hreflang code.

        :return str:
        """
        alternate_hreflangs = self.__get('alternate_hreflangs', item, [])
        output = ""
        if alternate_hreflangs:
            for lang, path in alternate_hreflangs:
                if urlparse.urlparse(path).netloc:
                    url = path
                else:
                    url = self._full_url(protocol, domain, path)
                output += REL_ALTERNATE_HREFLANG_SITEMAP_TEMPLATE.format(
                    **{'lang': lang, 'href': url}
                )
        return output

    def get_urls(self, page=1, site=None, protocol=None):
        """Get URLs."""
        # Determine protocol
        if self.protocol is not None:
            protocol = self.protocol
        if protocol is None:
            protocol = 'http'

        # Determine domain
        if site is None:
            if Site._meta.installed:
                try:
                    site = Site.objects.get_current()
                except Site.DoesNotExist:
                    pass
            if site is None:
                raise ImproperlyConfigured(
                    "To use sitemaps, either enable the sites framework "
                    "or pass a Site/RequestSite object in your view."
                )
        domain = site.domain

        urls = []
        for item in self.paginator.page(page).object_list:
            loc = self._full_url(
                protocol,
                domain,
                self.__get('location', item)
            )
            url_info = {
                'location': loc,
                'lastmod': self.__get('lastmod', item, None),
                'changefreq': self.__get('changefreq', item, None),
                'priority': self.__get('priority', item, None),
                'alternate_hreflangs': self._render_alternate_hreflangs(
                    protocol,
                    domain,
                    item
                ),
            }

            urls.append(url_info)
        return urls



__title__ = 'django-qartez'
__version__ = '0.7'
__build__ = 0x000007
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'

from django.utils import version

if isinstance(version.get_version(), float) and version.get_version() < 1.9:
    from .sitemaps import *



__title__ = 'qartez.constants'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__all__ = ('REL_ALTERNATE_HREFLANG_SITEMAP_TEMPLATE',)

# Tiny bit of XML responsible for rendering the alternate hreflang code
REL_ALTERNATE_HREFLANG_SITEMAP_TEMPLATE = """
<xhtml:link rel="alternate" hreflang="{lang}" href="{href}"/>
"""



from django.conf import settings

from . import defaults

__title__ = 'qartez.conf'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__all__ = ('get_setting',)


def get_setting(setting, override=None):
    """
    Get a setting from ``qartez`` conf module, falling back to the default.

    If override is not None, it will be used instead of the setting.
    """
    if override is not None:
        return override
    if hasattr(settings, 'QARTEZ_{0}'.format(setting)):
        return getattr(settings, 'QARTEZ_{0}'.format(setting))
    else:
        return getattr(defaults, setting)



import os
from setuptools import setup, find_packages

try:
  readme = open(os.path.join(os.path.dirname(__file__), 'README.rst')).read()
except:
  readme = ''

template_dir = "src/qartez/templates/qartez"
templates = [os.path.join(template_dir, f) for f in os.listdir(template_dir)]

version = '0.7'

setup(
    name='django-qartez',
    version=version,
    description=("Additional XML sitemap functionality for Django"),
    long_description=readme,
    classifiers=[
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.3",
        "Environment :: Web Environment",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "Development Status :: 5 - Production/Stable",
        "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
        "License :: OSI Approved :: GNU Lesser General Public License v2 or later (LGPLv2+)",
    ],
    keywords='xml sitemaps, images sitemaps, django, app, python',
    author='Artur Barseghyan',
    author_email='artur.barseghyan@gmail.com',
    license='GPL 2.0/LGPL 2.1',
    url='https://bitbucket.org/barseghyanartur/django-qartez',
    package_dir={'': 'src'},
    packages=find_packages(where='./src'),
    package_data={'qartez': templates},
    include_package_data=True,
    install_requires=[
        'six>=1.1.0',
        'django-nine>=0.1.7'
    ],
    tests_require=[
        'radar>=0.3',
    ]
)



import unittest

from ..tests.test_sitemaps import *

__title__ = 'qartez.tests'
__author__ = 'Artur Barseghyan'
__copyright__ = 'Copyright (c) 2013-2014 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'


if __name__ == '__main__':
    unittest.main()
